p#-*- coding: ISO-8859-15 -*-
#-------------------------------------------------------------------------------
#
# $Id: version.py 4819 2018-11-02 05:32:25Z jeanluc $ 
#
# Copyright 2016 Jean-Luc PLOIX
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-------------------------------------------------------------------------------
'''
Created on 20 nov. 2013

@author: jeanluc
'''
VERSION_MAJOR = 19
VERSION_MINOR = 12
VERSION_MAINTENANCE = 30
VERSION_BUILD = 1
TAG = "a"

# history # 

# 19.1.0a1  creation of metaphor_nntoolbox project
# 18.6.13.3  update of cython to 0.28.5
# 18.6.13.2  update of cython to 0.25
# 18.6.13.1  return to targz build format
# 18.6.13.0  return to cython use
# 18.6.12.1  add tokenlib in required
# 18.6.11.0  repeat to get dev and std versions simultaneously
# 18.6.8.2   modify version display
# 18.6.8.1   add rstrip in file reading in manageApiOptionsResult
# 18.6.8.0   add rstrip in file reading in manageApiOptionsResult
# 18.6.7.1   result axis format to "'#,##0.00'"
# 18.6.7.0   adjust docker
# 18.6.6.4   fix bug in multiusage
# 18.6.6.3   fix bug in multiusage
# 18.6.6.2   fix bug in multiusage
# 18.6.6.1   fix bug in multiusage
# 18.6.6.0   compile docker
# 18.5.28.0  new get_results_from_smiles in api.run
# 18.5.26.0  remise en ordre des sauvega    rdes wush
# 18.5.25.3  idem
# 18.5.25.2  fix bug in usage NN without id and target
# 18.5.25.0  usage decimal piloted by target string
# 18.5.24.3  back to cython
# 18.5.24.2  back to cython
# 18.5.24.1  no cython
# 18.5.24.0  use wheel everywhere
# 18.5.23.2  version beck to cython
# 18.5.23.1  version no_cython
# 18.5.22.0  remove versions in install requires
# 18.5.19.0  full installrequire
# 18.5.18.1  change modeltemplate to start with 'zzz'
# 18.5.18.0  fix bug in indexing files in GM create model
# 18.5.17.1  fix bug in default init with @ argfile calling
# 18.5.17.0  fix bug in finishExcel

# 18.5.16.1  fix bug in finishExcel
# 18.5.16.0  fix dockerfiles nn1_base
# 18.5.15.8  still working on dependencies
# 18.5.15.4 remove dependencylinks from setup
# 18.5.15.1  remove import futures from nntoolbox.setup.py
# 18.5.15.0  remove import futures
# 18.5.12.1  fix bug SF_CODE -> SF_CCODE in _driver
# 18.5.12.0  OK nn1 api interface and demo jupyter
# 18.4.21.4  OK test argfile creation
# 18.4.21.3  fix nntoolbox.utils.getListFromStr to accept None input
# 18.4.21.0  display LOO inspected
# 18.4.20.0  clarify init options
# 18.4.19.0  random init for trainjob back to the old method
# 18.4.18.7  cosmetic
# 18.4.18.6  cosmetic
# 18.4.18.2  send to sys.stderr error messages that stop
# 18.4.18.1  cosmetic display
# 18.4.18.0  fixing display with berbose and verbosexls
# 18.4.17.9  modif utils.callerView to remove /home/ from displayed filenames
# 18.4.17.8  modif utils.callerView to remove /home/ from displayed filenames
# 18.4.17.3  fix bug DEMO_DEBUG_FOLDER
# 18.4.17.2  fix bug results display internal folder
# 18.4.17.1  fix bug "/docker" in nntoolbox/utils
# 18.4.17.0  change internal excvhange folder from /docker to /home
# 18.4.16.3  full version display for VERSION fixed
# 18.4.16.0  full version display for VERSION fixed
# 18.4.15.3  ful version display for VERSION
# 18.4.15.2  modernize monalrecords
# 18.4.15.1  init xml=None in getModelDict
# 18.4.15.0  supp modeldata from nnx
# 18.4.14.0  check modelCount in existing training models
# 18.4.12.0  extend search input files on all children of shared folder
# 18.4.11.3  reading hostpath from docker in host.pth ...
# 18.4.11.0  reading hostpath from docker in host.pth
# 18.4.10.3  OK fullH
# 18.4.10.2  check csv data foermat
# 18.4.10.1  change static lib file names more specific
# 18.4.10.0  fix bug erase tempdir before compiling
# 18.4.9.4   supp '-D_FILE_OFFSET_BITS=64' in compile dll
# 18.4.9.3   declarations extern en C 3
# 18.4.9.2   declarations extern en C 2
# 18.4.8.1   declarations extern en C
# 18.4.8.0   track bug testH
# 18.4.6.3   main tra    in back to parallel
# 18.4.6.2   init seed in LOO indexed on init and no longer on training example
# 18.4.6.1   train with H atoms
# 18.4.6.0   accept H atoms froim smiles
# 18.4.5.2   fix loo save
# 18.4.5.0   fix usage multi display
# 18.4.4.14 reprise usage gm

# 18.4.4.5  OK usage with few parameters
# 18.4.4.0   sqlfilename and sqlLOOfilename in excel file
# 18.4.3.6   fix resultFrame call and recall
# 18.4.3.4   fix loo decor and resultFrame recall
# 18.4.3.3   fix decor empty
# 18.4.3.2   fix decorfile
# 18.4.3.1   fix bug in getSaveDictMulti
# 18.4.3.0   fix bug loo only
# 18.3.29.0  set usage mode - 1
# 18.3.26.0  debug tootest + modif random initia    lization in parallel jobs
# 18.3.24.0  debug indexation des lootrain sur excel
# 18.3.23.0  debug test results indexings
# 18.3.22.5  debug normalisation des modèles en test (nn1 et gm)
# 18.3.22.2  debug gm test
# 18.3.22.2  debig lootest for gm projects - dont work for nn projects
# 18.3.21.1  ajout de '-D_FILE_OFFSET_BITS=64' dans les azrguments de compilation du module dynamique
# 18.3.21.0  fix bug in seed init in parallel jobs (std anf LOO-BS)
# 18.3.20.3  fix bug on setidentifier in C code
# 18.3.20.0  bug on gm molecules display - use #define _FILE_OFFSET_BITS  64 in dll C code - nn1 OK (dev) on train, test, loo for nn and gm
# 18.3.16.0  intro of chem_gm in nn1 requireùents
# 18.3.15.3  fix library compile, including monal lib
# 18.3.15.0  fix sql selection results, multi compile
# 18.3.14.0 blocksize variable in multimodel compile - display local seed in training 
# 18.3.5.1 no input type checking
# 18.3.5.0 optimisation C code
# 18.3.2.1 fix bug in compile multimodels aftder modif of compile_lib
# 18.3.2.0 fix LOO bug in loo models selection
# 18.3.1.0  modif code C for memory blocking
# 18.2.9.0 rewrite inputs selection
# 18.2.8.1 
# 18.2.8.0  revisit input selection wth gramm-schmidt
# 18.2.2.1  follow nntoolbox 18.2.2.1  relax openpyxl version requirement
# 18.1.31.0
# 7.0.2.25 extract _driverlib from cython
# 7.0.2.22 extract monal/datareader/fileInspect from cython
# 7.0.2.21 preceeding without monal/model
# 7.0.2.19 preceeding + monal/model except _network, _node, _link, _modelfunc
# 7.0.2.18 preceeding + monal/model except _network, _node, _link
# 7.0.2.17 preceeding + monal/model except _network
# 7.0.2.16 preceeding + monal/model
# 7.0.2.15 preceeding + monal/library
# 7.0.2.14 preceeding + monam/driver in cythonlist except networkfactory, drivermultidyn, parallelJobs
# 7.0.2.13 preceeding + monam/driver in cythonlist except networkfactory, drivermultidyn
# 7.0.2.12 preceeding + monam/driver in cythonlist except networkfactory
# 7.0.2.11 preceeding + monam/driver in cythonlist
# 7.0.2.10 add all monal/datareader monal/lcode monal/util*.py to cythonlist except monal/util/utils.py
# 7.0.2.8  add all monal/datareader monal/lcode monal/model*.py to cythonlist
# 7.0.2.7  add all monal/datareader monal/lcode *.py to cythonlist
# 7.0.2.6  add all monal/datareader *.py to cythonlist
# 7.0.2.5  add all monal *.py to cythonlist
# 7.0.2.4  follow nntoolbox 1.0.2.4  full cython on nntoolbox
# 7.0.2.3  follow nntoolbox 1.0.2.2 cython on nntoolbox ote filetoolbox 
# 7.0.2.2  follow nntoolbox 1.0.2.2  cython on nntoolbox
# 7.0.2.1  follow nntoolbox 1.0.2.1   pandas > 0.21.0 => set_value, get_value depreecated
# 7.0.2.0  basic woth no cython proptection
# 7.0.1.55 test of C extension in monal
# 7.0.1.54 test of C extension in monal

#  ...25  test setup & install
# 7.0.1.18 follow nntoolbox 1.0.1.15 test cleaner
# 7.0.1.17 follow nntoolbox 1.0.1.14 test cleaner
# 7.0.1.16 follow nntoolbox 1.0.1.13 test cleaner
# 7.0.1.15 follow nntoolbox 1.0.1.12 fix bug in cleanup
# 7.0.1.14 follow nntoolbox 1.0.1.11 fix bug in setup
# 7.0.1.13 follow nntoolbox 1.0.1.10 new setup
# 7.0.1.12 follow nntoolbox 1.0.1.9  fix bug in configparsertoolbox
# 7.0.1.11 follow nntoolbox 1.0.1.8  return to setup process
# 7.0.1.10 follow nntoolbox 1.0.1.7  no arguments at all 2
# 7.0.1.9  follow nntoolbox 1.0.1.6  no arguments at all
# 7.0.1.6  follow nntoolbox 1.0.1.5  fix bug test only
# 7.0.1.4  follow nntoolbox 1.0.1.4  bug excelutil color lev thresh
# 7.0.1.1  follow nntoolbox 1.0.1.1
# 7.0.1.0  follow nntoolbox 1.0.1.0  first OK of nn1 in docker
#  35 fix bug in monaltoolbox choicequestion
# 7.0.0.19 frix bug in util/utils
# 7.0.0.17 follow nntoolbox 1.0.0.23
# 7.0.0.16 follow nntoolbox 1.0.0.22 exception on clean files
# 7.0.0.14 follow nntoolbox 1.0.0.20
# 7.0.0.10 add 'magic' in required in setup
# 7.0.0.9  follow nntoolbox 1.0.0.20 clean filetoolbox
# 7.0.0.6  follow nntoolbox 1.0.0.19 fix install with test
# 7.0.0.3  follow nntoolbox 1.0.0.3 modify utils
# 7.0.0.2  follow nntoolbox 1.0.0.2 add average data on test chart
# 7.0.0.1  follow nntoolbox 1.0.0.1 fix bug in excelutils (index in test chart)
# 7.0.0.0  follow nntoolbox 1.0.0.0  OK first NN1
# 6.1.1.15 follow nntoolbox 0.0.0.6
# 6.1.1.14 follow nntoolbox 0.0.0.5
# 6.1.1.12 fix setup bug
# 6.1.1.11 follow nntoolbox 0.0.0.4
# 6.1.1.10 follow nntoolbox 0.0.0.4
# 6.1.1.0  test expand cythonlist 
# 6.1.0.49 OK setup with cythonlist -> remove py files 
# 6.1.0.33 test setup with cythonlist -> remove py files 
# 6.1.0.32 test setup 
# 6.1.0.31 test setup 
# 6.1.0.30 test setup 
# 6.1.0.29 test setup 
# 6.1.0.28 test setup 
# 6.1.0.27 test setup with getCythonList
# 6.1.0.26 publi 25 October 2017
# 6.1.0.25 publi October 2017
# 6.1.0.24 automate copy pyx
# 6.1.0.23 OK cythonize 5 files
# 6.1.0.21 OK cythonize 7 files
# 6.1.0.14 OK cythonize 3 files
# 6.1.0.13 OK cythonize 3 files
# 6.1.0.3..12  test x on cythonize
# 6.1.0.2  test 3 on cythonize
# 6.1.0.1  test 2 on cythonize
# 6.1.0.0  test 1 on cythonize

# 6.0.1.3  work on debug encoding files monaltoolbox debugOutput
# 6.0.1.2  for docker demo 6.1
# 6.0.1.1  change numpy version to 1.13.3
# 6.0.1.0  fix bug in configstr, central, decor
# 6.0.0.42 fix bug in sqlite_copy_db
# 6.0.0.41 new gm with env
# 6.0.0.40 fix bug in toolbox.yesnoquestion : raw_input
# 6.0.0.39 fix bug in toolbox.isint
# 6.0.0.38 fix bug in toolbox.isint
# 6.0.0.37 fix bug in util.utils.CCharOnlys with u nicode
# 6.0.0.36 OK py36
# 6.0.0.35 fix install setup: no lib compile ins install.
# 6.0.0.34 modif openpyxl requirement >=...
# 6.0.0.33 put back afterinstall in setup
# 6.0.0.31 debug
# 6.0.0.30 eliminate makdtemp in util.utils
# 6.0.0.29 add mkdirs after mkdtemp in util.utils
# 6.0.0.27 fix bug on compilemonallib in utils
# 6.0.0.26 fix bug on compilemonallib in utils
# 6.0.0.25 fix bug on compilemonallib in utils
# 6.0.0.24 fix bug on compilemonallib in utils
# 6.0.0.23 fix bug on xfile in toolbox
# 6.0.0.22 introduce crypt fonctionamity in utils for comple monal lib
# 6.0.0.21 Fix bug in data files to install
# 6.0.0.20 fix setup without compile monal lib
# 6.0.0.19 re-add compilemonallib in compile module, from crypted files
# 6.0.0.18 fix haspackage in setuptoolbox
# 6.0.0.17 fix haspackage in setuptoolbox
# 6.0.0.16 fix haspackage in setuptoolbox
# 6.0.0.15 test install docker
# 6.0.0.14 eliminate 'pip-build' content monalpath
# 6.0.0.13 fix install bug in python 3
# 6.0.0.11 add wirites in installmonal.txt
# 6.0.0.10 fix install bug bad location & erase failing
# 6.0.0.8  fix install bug bad location & erase failing
# 6.0.0.8  fix print bug in lineartoolbox
# 6.0.0.7  remove fu ture from install requirement in setup
# 6.0.0.6  bug in setuptoolbox.compilemonallib test
# 6.0.0.5  fix utf-8 errors in C codes crypt
# 6.0.0.4  fix install bug in C code crypt
# 6.0.0.3  fix install bug on environ
# 6.0.0.2  test dist PyPi
# 6.0.0.1  test dist PyPi
# 6.0.0.0  Python 3.6 
# 5.0.0.2  modif fut ures in fu ture in setup required
# 5.0.0.1  fix bug in _driver.TransferEx.mappingtransferleverages
# 5.0.0.0  Code Python 3.1
# 4.5.0.2  fix bug in setup.py templatelist ->->->->->->->->->->->->Publi
# 4.5.0.1  modif MANIFEST.in to eliminate uncrypted files
# 4.5.0.0  set crypted template sources
# 4.4.1.4  demo avec bootstrap OK
# 4.4.1.3  adjust CORRECTED _FIELD & create dbField in monalconst
# 4.4.1.2  intro onSpecialResult in driver
# 4.4.1.1  intro loo in api demo
# 4.4.1.0  reorg gmx for eliminate config

# 4.4.0.2  ???
# 4.4.0.1  add loadFrom in monal.util.toolbox
# 4.4.0.0  last before loo and bootstrap development
# 4.3.2.37 modify monal.excelsource.getcustomproperty
# 4.3.2.36 add SUBSTITUTE_FIELD in monalconst
# 4.3.2.35 modify to CORRECTED_ FIELD in monalconst
# 4.3.2.34 add CORRECTEDFIELD in monalconst
# 4.3.2.33 test sql_numpy
# 4.3.2.32 cosmetic in util.utils
# 4.3.2.31 heu ?
# 4.3.2.30 fix bug in driver.transferex 
# 4.3.2.28 add getcustomproperty in datareader.excelsource
# 4.3.2.27 add hasOneOf in util.toolbox
# 4.3.2.26 add enrichFilename in util.utils
# 4.3.2.25 ??
# 4.3.2.24 modif repr
# 4.3.2.23 ??
# 4.3.2.22 modif .__repr__
# 4.3.2.21 add chmod to compilemonallib
# 4.3.2.20 ecclude *.a in setup
# 4.3.2.19 
# 4.3.2.18 fix bug monaltoolbox
# 4.3.2.17 display of seconds
# 4.3.2.16 supp space before ":" in display
# 4.3.2.15 code version 46
# 4.3.2.14 ???
# 4.3.2.13 monal/util/utils.backupfile modify to return true if file do not exists
# 4.3.2.12 cosmetic
# 4.3.2.11 cosmetic
# 4.3.2.10 upgrade codeversion to recompile models eliminating default connect values
# 4.3.2.9  cosmetic
# 4.3.2.8 fix bug in _modellib
# 4.3.2.7  fix bug in _modellib
# 4.3.2.6  add substitute of scipy
# 4.3.2.5  add tabledisplatdict in monalconst
# 4.3.2.4  bug fix in modellib import
# 4.3.2.3  contournement de scipoy dans _modellib. rest a faire dans mplUtils
# 4.3.2.2  _drivermultidyn optimize check presence of scipy
# 4.3.2.1  suppr scipy requirement in setup. _driver IC computation and train from optimize
# 4.3.2.0  suppr scipy requirement in setup. IC computation to set as independant of scipi.stats.t

# 4.3.1.92 cosmetic changes
# 4.3.1.91 modify DriverMultiDyn.repr to introduce veryshort level
# 4.3.1.90 cosmetic changes
# 4.3.1.89 utils.backupfile revisited -> xxx_backup_.ext
# 4.3.1.88 pretty setup +modif codeconst.__version__ 42 + utils.backupfile revisited
# 4.3.1.87 add setuptoolbox in monal.util to separate frol toolbox
# 4.3.1.86 bug fix in train.c (=! -> !=) and move install utilities from setup.py to toolbox.py
# 4.3.1.85 codeversion update
# 4.3.1.84 bug fix in tain.c earlyHii
# 4.3.1.83 monal.util.utils backupfile modif
# 4.3.1.82 gmapidemo manage OKcodeversion
# 4.3.1.81 modif calcul de deltash in train.c par diff des sommes de hii => modif codeconst.__version__ 39
# 4.3.1.80 modif calcul de deltash in train.c => modif codeconst.__version__ 38
# 4.3.1.79 enrichment of toolvox/IndexFilename -> add "last" param
# 4.3.1.78 Ok for api use existing models
# 4.3.1.77 modif gmapidemo -> useexisting added param
# 4.3.1.76 fix bug in LibManager.codeversionOK 
# 4.3.1.75 
# 4.3.1.74 stabilisation
# 4.3.1.73 stabilisation
# 4.3.1.72 modif monal/util/IndexFilename
# 4.3.1.71 modif monal/util/IndexFilename
# 4.3.1.70 isnan bug fixed
# 4.3.1.69 create monal/api
# 4.3.1.68 add xlsm as accepted filetype in excelsource
# 4.3.1.67 add gettitles in datareader/excelsource
# 4.3.1.66 change read_sql with read_sql_query in sqlite_numpy.getDataFrame
# 4.3.1.65 minor cosmetic bugs fixed
# 4.3.1.64 minor cosmetic bugs fixed
# 4.3.1.63 work on datafields and testfields
# 4.3.1.62 put datafields in datareader/excelsource
# 4.3.1.61 optimize speed
# 4.3.1.60 add IndexFilename in monal/util/toolbox
# 4.3.1.59 modif setup install_requires to be coherent with chem_gm
# 4.3.1.58 adjust grad and accessories length with longParamCount
# 4.3.1.57 supp adptParams and adaptDispersion in _Driver 
# 4.3.1.55 bug fix in toolbox
# 4.3.1.54 speed up usage
# 4.3.1.53 impose epoches as integer in -paralleljobs.onTreturnResult
# 4.3.1.52 adjust train, transferOld
# 4.3.1.51 mod C code add superOut in trainbase
# 4.3.1.50 mod multidyn for moduleType
# 4.3.1.49 improve _linkstr in util/toolbox
# 4.3.1.48 add _linkstr in util/toolbox
# 4.3.1.47 add dict attribute in _drivermultidyn
# 4.3.1.46 add xlwt in install required
# 4.3.1.45 use no lib in gm usage demo
# 4.3.1.44 rewrite raw_input environment
# 4.3.1.43 load readline in monaltoolbox for raw_input
# 4.3.1.42 short repr for multimodel
# 4.3.1.41 update valuequestion to understand "y" as default
# 4.3.1.40 update yesnoquestion for "hit any key to quit"
# 4.3.1.39 add forceyes in monaltoolbox.yesnoquestion and forcedefault in monaltoolbox.valueQuestion
# 4.3.1.38 OK dll model 
# 4.3.1.36 fix bug in 
# 4.3.1.35 move datasource.py from GM//************///
# 4.3.1.34 fix bug de getConfiguration dans les codes C 
# 4.3.1.33 fix bug de moduletype dans les codes C (;)
# 4.3.1.32 introduction de moduletype dans les codes C
# 4.3.1.31 transfert de getFrame dans excelsource
# 4.3.1.30 utilisation de ctypes.create_string_buffer avec dim+1 tjs
# 4.3.1.29 remise libmonal.a
# 4.3.1.28 debug install
# 4.3.1.26 add extra_preargs in monal/util/utils.compile_lib
# 4.3.1.25 debug install
#   .
#   .
#   .
# 4.3.1.13 debug install
# 4.3.1.12 suppress libmonal.a de l'install
# 4.3.1.11 -fPIC option suppressed for monal.a lib
# 4.3.1.10 library prefix 'lib' in linux
# 4.3.1.9  fix -fPIC option in lib compile for installation
# 4.3.1.8  fix -fPIC option in module compile (necessary for ubuntu)
# 4.3.1.7  sprintf "" fixed in interfacetrainingdll.c
# 4.3.1.6  mise au pt creation modele serie pour ubuntu
# 4.3.1.5  'jurgb.txt''jurfr.txt''licgb.txt''licfr.txt'
# 4.3.1.3  GetJurComment lower in codewriter. 'jurgb.txt''jurfr.txt''licgb.txt''licfr.txt'
# 4.3.1.2  fix monaltoolbox bug
# 4.3.1.1  adapt to gm api demo + install requires
# 4.3.1.0  adapt to gm api demo
# 4.3.0.1  create test libmanager
# 4.3.0.0  retour calcul delta sh separe de delta rank
# 4.2.0.6
# 4.2.0.5  OK with GM
# 4.2.0.4  separatikon des tests
# 4.2.0.1  fix bug configfile getappliconfig getapplidatabase 
# 4.2.0.0  OK auto upgrade
# 4.1.1.57 add "--user" if is User in install
# 4.1.1.56 remove --user option when virtual
# 4.1.1.55 fix bug in util.utils.getancestor
# 4.1.1.53 test new upgrademgr

# 4.1.1.52 test new upgrademgr
# 4.1.1.51  upgrade isVirtual and GetFile for iMac
# 4.1.1.49  cosmetic in debug file
# 4.1.1.48  cosmetic in debug file
# 4.1.1.46  cosmetic in debug file
# 4.1.1.44  introduce date and version in debug file
# 4.1.1.43  introduce date and version in debug file
# 4.1.1.41  modif to "--debug" setup option
# 4.1.1.40  modif to "debug" setup option
# 4.1.1.39  ajout de --debug option
# 4.1.1.36  mise au point setup - debug=0

# 4.1.1.19  extend graphmachineconstant.getfile

#  4.1.1.0  suppress call to exports sybol in dll compiling
#  4.1.0.3  move monal.a creation from gra^hmachine install to monal install
#  4.1.0.2  move monal.a location to monal.lcode and create it during installation
#  4.1.0.1  move sqlite_numpy.py from graphmachine.component to monal.util
#  4.1.0.0  OK version release 16/06/2016
#  4.0.0.33 random seed again
#  4.0.0.32 randaom seed in parallel jobs while computing LOO
#  4.0.0.31 OK multitrain, localtrain
#  4.0.0.30 OK multitrain
#  4.0.0.29 OK multitrain
#  4.0.0.28 le 07/06/2016
#  4.0.0.27 le 06/06/2016
#  4.0.0.26 05/06/2016
#  4.0.0.25 pb de Pypi
#  4.0.0.24 fix delta sh avec early hii sum
#  4.0.0.23  fix C code training
#  4.0.0.22  after register version de reference
#  4.0.4.21 fix CChar in module names
#  4.0.4.20 test upgrade
#  4.0.4.19 test upgrade
#  4.0.4.18 fix small bugs
#  4.0.4.17 getapplibase extended in util.utils
#  4.0.4.15 add specifiers in setup.py
#  4.0.4.13 change download_urls in setup.py
#  4.0.4.12 test PyPI upload
#  4.0.4.9  new console interface for gm  
#  4.0.4.8  create CRITERION_TYPE_STR in monalconst
#  4.0.4.7
#  4.0.4.6  
#  4.0.4.4  work on event management
#  4.0.4.3  cleaning dialogs
#  4.0.4.2  multitrain management of training failure
#  4.0.4.1  multitrain proptect from writer None
#  4.0.4.0  cosmetic
#  4.0.3.7  reorganize modelcontainer
#  4.0.3.6  OK El Capitan + XCode 7.3
#  4.0.3.5  fix CCharsOnly bug
#  4.0.3.4  adapt to GM 3.0.0.3 & 4
#  4.0.3.3  adapt to GM 3.0.0.2
#  4.0.3.2  adapt to GM 3.0.0.1
#  4.0.3.1  introducing P-Train
#  4.0.2.43 
#  4,0,2,42 fix LOO bugs
#  4.0.2.41 fix network display with alternate columns
#  4.0.2.40 
#  4.0.2.39 
#  4.0.2.38 intro py 2app
#  4.0.2.37 fix install
#  4.0.2.36 Separation __init__.py et version.py
#  4.0.2.35 NoSectionError  
#  4.0.2.34 add FileConfifParser
#  4.0.2.33 version commune P2/P3
#  4.0.2.31 en cours
#  4.0.2.30 setup epure
#  4.0.2.29 maxworkers 0
#  4.0.2.28 modif import concurrent.fu tures
#  4.0.2.27 maxworkers programmable
#  4.0.2.25 trainBS selection upgrade
#  4.0.2.24 fix memory leak with np.load
#  4.0.2.23 fix memory leak with np.load
#  4.0.2.22 fix multiLOO sort bug
#  4.0.2.21 en cours super train
#  4.0.2.20 multiLOO
#  4.0.2.19 track on node.deleteLink consequences
#  4.0.2.18 track on node.deleteLink consequences
#  4.0.2.17 node.deleteLink fixed
#  4.O.2.16 fix training display 
#  4.0.2.15 modif DISPFIELDNAMES
#  4.0.2.14 toolbox raw(str)
#  4.0.2.13 noms accentues
#  4.0.2.11 iso multiple
#  4.0.2.10 fix non ASCII char in model names in C code
#  4.0.2.9 fix iso bug
#  4.0.2.8 annotation folles
#  4.0.2.7 
#  4.0.2.6 formula
#  4.0.2.5 fix LOO inittype
#  4.0.2.4 fix charge vs grade et garde vs ring
#  4.0.2.3 bugs
#  4.0.2.2 fix iso bug
#  4.0.2.0 ajout gestion des modeles dans GM
#  4.0.1.9 integration getPRESS
#  4.0.1.8 process interrupt, OK Bootstrap
#  4.0.1.7 large database with smiles beginning with "(" 
#  4.0.1.6 Bug init random LOO OK
#  4.0.1.5 Test sur les sommes des hii pour la selection de modele
#  4.0.1.4 Multiinit LOO with PRESS selection
#  4.0.1.3 multinit for LOO
#  4.0.1.2 creating modules shorten
#  4.0.1.1 loading single modules too long fixed
#  4.0.1   LOO bug fixed
#  4.0.0a3 LOO bug fixed almost
#  4.0.0a2 
#  4.0.0a1 small bugs
#  4.0.0a0 OK first parallel
#  3.3.2.x dans la branche referenceGM
#  3.3.2.0 OK avant parallel
#  3.3.1a14 fix bug memWeight for trainBS-LOO
#  3.3.1a13 fix bug on interfacetraindll.c getCost
#  3.3.1a12 calcul des LOOleverages
#  3.3.1a11 apprentissages special avec poids de depart specifie
#  3.3.1a10 fix auto update
#  3.3.1a9 fix bug test set
#  3.3.1a8 supp les install_requires dans le setup.py
#  3.3.1a7 mise a jour auto
#  3.3.1a6 inclusion directe des *.c dans les modules d'apprentissage
#  3.3.1a5 securisation de "mathplus.c" et "nttoolbox.c"
#  3.3.1a4 amemioration de "restart" dans monal.util.utils.py : impression des params
#  3.3.1a3 affinement apres update
#  3.3.1a2 ajout de "restart" dans monal.util.utils.py
#  3.3.1a1 alignement sur la sous version de graphmachine (premiere sous version avec le chaeck for update complet)
#  3.3.1a0 deplacement des info codeversion vers les procedures d'initialisation
#  3.3.0a8 ajout codeversion
#  3.3.0a7 corr bug usage multi  -> non pour leverage
#  3.3.0a6 corr bug usage multi
#  3.3.0a5 en usage, leverage par methode monari
#  3.3.0a4 config auto
#  3.3.0a3 fusion de la crÃ©ation de modele et de l'enregistrement du code C; destruction rapide des modeles en memoire et remplacement par des leurres
#  3.3.0a2 accompagne GM 1.3.0a2
#  3.3.0a1 dans utyil/utils/getapplicfg, recopie du fichier cfg s'il est plus vieux que la candidat model
#  3.3.0a0 Utilisation des modules dll aussi pour modÃ¨les simples
#  3.2a05  Sauvegarde modules container en dll
#  3.2a04  Use train.c in library
#  3.2  Use abstract compilers for saving in python module format
#  3.1  OK pout GM 1.0
#  3.0

__version__ = "%d.%d.%d.%d" % (VERSION_MAJOR, VERSION_MINOR, VERSION_MAINTENANCE, VERSION_BUILD)  
VERSION3 = "%d.%d.%d" % (VERSION_MAJOR, VERSION_MINOR, VERSION_MAINTENANCE)
VERSION2 = "%d.%d" % (VERSION_MAJOR, VERSION_MINOR)
try:
    if TAG:
        __version__ = "{0}.{1}{2}".format(VERSION3, TAG, VERSION_BUILD)
except: pass
version = lambda : __version__

def distname(path="", ext=".tar.gz"):   
    import os
    st = "monal-%s%s"% (__version__, ext)
    if len(path):
        st = os.path.join(path, st)
    return st

if __name__ == "__main__":
    print(__version__)



#-*- coding: ISO-8859-15 -*-
#===============================================================================
# $Id: _modellib.py 4738 2018-03-26 05:30:12Z jeanluc $
#  Module _modellib.py
#  Projet MonalPy
#
#  modeles fondes sur les libraries partagees
#
#
# Copyright 2016 Jean-Luc PLOIX
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#===============================================================================

#from __fu ture__ import absolute_import
import os
import numpy as np
from ctypes import create_string_buffer, byref, c_int, c_double, c_void_p
import math
#from nntoolbox._2and3 import isstr
from six import string_types
try:
    from scipy.stats.t import interval
except:
    from ..util.monaltoolbox import interval
from ..model import ModelBase
from ..Property import Property, Properties
from ..lcode import codeconst as CC
from ..library.libmanager import LibManager, LibraryManagerError


class ModelLibError (Exception): 
    """ModelLib Exception.
    """
    pass

@Properties(("confidencelevel", "extraweights", "extradispersions", 
             "LOOweights", "BSweights", "LOOweightsMulti"), False)
class ModelLib(ModelBase, LibManager): 
    """class ModelLib
    Gestionnaire des modeles provenant de librairies dynamiques (dll, so, dylib)
    """
    
    def __init__(self, libfilename, owner=None, name="", source=None, paramfile=None):
        ModelBase.__init__(self, owner, name)
        LibManager.__init__(self, libfilename)
        self.weights = None
        self._LOOweightsMulti = None
        self._LOOweights = None
        self._BSweights = None
        if source:
            if isinstance(source, string_types):
                with np.load(source, allow_pickle=True) as sourceloc:
                    if "propertyname" in sourceloc.files:
                        self.propertyName = str(sourceloc["propertyname"])
            else:
                if "propertyname" in list(source.keys()):
                    self.propertyName = str(source["propertyname"])
        self.loadParameters(paramfile if paramfile else source)
            
    def __del__(self):
        LibManager.__del__(self)
        ModelBase.__del__(self)

    def __enter__(self):
        return self
    
    def __exit__(self, etype, value, traceback):
        self.__del__()
    
    def __repr__(self):
        return self.getDescription()
    
    def getDescription(self, keeplist=False):  #, extended=False
        lst = [("From module %s"% self.moduleName)]
        lst.append("module type %s"% self.moduleType)
        lst.append("code version/current %s/%s"% (self.codeVersion, CC.__useversion__))
        lst.append("created %s"% self.created)
        if self.mark: #getStrPropertyFromLib("mark"):
            lst.append("mark %s"% self.getStrPropertyFromLib("mark"))
        lst.append("Molecule %s"% self.modelName)
        #lst.append("Module %s"% self.moduleName)#getStrPropertyFromLib("moduleName"))
        lst.append("Smiles %s"% self.smiles)#getStrPropertyFromLib("smiles"))
        lst.append("Train base %s"% self.trainBase)#getStrPropertyFromLib("trainbase"))
        lst.append("Configuration %s"% self.configuration)
        if self.propertyName:
            lst.append("propertyName %s"%self.propertyName)
        lst.append("paramcount %d"%self.paramCount)
        lst.append("dimension %d"%self.dimension)
        lst.append("hidden %s"%self.hidden)
        if self.BSCount:
            lst.append("BSCount %s"%self.BSCount)
        if self.LOOCount:
            lst.append("LOOCount %s"%self.LOOCount)
        if self.trainingStddev:
            lst.append("trainingStddev %s"%self.trainingStddev)
        if self.baselen:
            lst.append("baselen %s"%self.baselen)
#         if extended and self.configuration:
#             connect, central, classif, hidden = ReverseConfigString(self.configuration)
        if keeplist:
            return lst
        return "\n\t".join(lst)
    
    @property
    def modelName(self):
        return self.getStrPropertyFromLib("modelName").decode("ISO-8859-15")
    
    @Property
    def dimension(self): # LibManager
        return self.lib.dimension()
    
    @property
    def configuration(self):
        return self.getStrPropertyFromLib("configuration")
    
    @property
    def smiles(self):
        return self.getStrPropertyFromLib("smiles")
    
    @property
    def trainBase(self):
        return self.getStrPropertyFromLib("trainbase")
    
    @property
    def BSCount(self):
        if not hasattr(self, "_BSweights") or self._BSweights is None:
            return 0
        return self.BSweights.shape[0]
    
    @property
    def LOOCount(self):
        if not hasattr(self, "_LOOweights") or self._LOOweights is None:
            return 0
        return self.LOOweights.shape[0]
    
    def extraList(self):
        lst = []
        if hasattr(self, "_BSweights"):
            lst.append("BS")
        if hasattr(self, "_LOOweights") or hasattr(self, "_LOOweightsMulti"):
            lst.append("LOO")
        return lst
    
    def transferExtra(self, extratype=""):  #, axis=0):
        lst = []
        if (extratype == "LOO") and hasattr(self, "_LOOweightsMulti") and (self.LOOweightsMulti is not None):
            res = np.asarray([[self.transfer(weights) for weights in target] for target in self.LOOweightsMulti])
            return res
            #return np.mean(res, axis=axis)
            
        target = None
        if (extratype in ("", "BS")) and hasattr(self, "_BSweights"):
            target = self.BSweights
        elif (extratype == "LOO") and hasattr(self, "_LOOweights"):
            target = self.LOOweights
        if target is not None:
            lst = [self.transfer(weights) for weights in target]
        return np.asarray(lst)
    
#     def transferBS(self):  # DriverLib
#         return self.transferExtra("BS")
        
#     def transferLOO(self):  # DriverLib
#         return self.transferExtra("LOO")
        
    @Property
    def trainingStddev(self): # ModelLib
        no = self.lib.outputCount()
        target = np.zeros((no,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        self.lib.getstddev(target_p)
        if no == 1:
            return target[0]
        else:
            return target
    @trainingStddev.setter
    def trainingStddev(self, value): # ModelLib
        #no = self.lib.outputCount()
        source = c_double(value)
        return self.lib.setstddev(source)

    @Property
    def baselen(self): # ModelLib
        return self.lib.getbaselen()
    @baselen.setter
    def baselen(self, value): # ModelLib
        self.lib.setbaselen(int(value))
    
    @Property
    def params(self): # ModelLib
        nw = self.lib.getparams(None)
        target = np.zeros((nw,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        res = self.lib.getparams(target_p)
        if res:
            return None
        return target
    @params.setter
    def params(self, value): # ModelLib
        source = np.asarray(value)
        try:
            ll = source.shape[0]
        except IndexError:
            print("Error in params setter: source : %s"%source)
            raise
            
        source_p = c_void_p(source.ctypes.data)
        res = self.lib.setparams(source_p, c_int(ll)) 
        if res == -1:
            dim = self.lib.lasterror(None) 
            rtyp = create_string_buffer(dim + 1) 
            res = self.lib.lasterror(byref(rtyp))
            if not res: 
                raise ModelLibError(rtyp.value)
            raise ModelLibError("Unknown error")
    
    @Property
    def halfdispersion(self): # ModelLib
        nd = self.lib.dimension()
        nw = self.lib.paramCount(None)
        target = np.zeros((nd*nw,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        res = self.lib.gethalfdispersion(target_p)
        if res:
            return None
        target = np.reshape(target, (nw, nd))
        return target
    @halfdispersion.setter
    def halfdispersion(self, value): # ModelLib
        nd = self.lib.dimension()
        nw = self.lib.paramCount(None)
        source = np.asarray(value)
        if source.ndim == 1:
            ll2 = len(source)
            ll = int(math.sqrt(ll2))
            if (ll2 != ll*ll) or not (ll in [nw, nd]):
                raise ModelLibError("Bad dimension in dispersion setter")
        elif source.ndim == 2:
            ll, ld = source.shape
            if (ll != ld) or not (ll in [nw, nd]):
                raise ModelLibError("Bad dimension in dispersion setter")
            source = source.reshape(ll*ld,)
        source_p = c_void_p(source.ctypes.data)
        self.lib.sethalfdispersion(source_p, c_int(ll))
    
    @Property
    def dispersion(self): # ModelLib
        nw = self.lib.paramCount(None)
        #nw = self.lib.dimension()
        target = np.zeros((nw*nw,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        res = self.lib.getdispersion(target_p)
        if res:
            return None
        target = np.reshape(target, (nw, nw))
        return target
    
    @dispersion.setter
    def dispersion(self, value): # ModelLib
        nw = self.lib.paramCount(None)
        nd = self.lib.dimension()
        source = np.asarray(value)
        if source.ndim == 1:
            ll2 = len(source)
            ll = int(math.sqrt(ll2))
            if (ll2 != ll*ll) or not (ll in [nw, nd]):
                raise ModelLibError("Bad dimension in dispersion setter")
        elif source.ndim == 2:
            ll, ld = source.shape
            if (ll != ld) or not (ll in [nw, nd]):
                raise ModelLibError("Bad dimension in dispersion setter")
            source = source.reshape(ll*ld,)
        source_p = c_void_p(source.ctypes.data)
        self.lib.setdispersion(source_p, c_int(ll))
    
    def loadParameters(self, source): # ModelLib
        if source is None: return
        if isinstance(source, string_types):
            #chargement binaire
            try:
                with np.load(source) as source1:
                    res = self.loadFromDict(source1) or self.loadWeightsASCII(source, self) or self.loadWeightsXML(source, self)
#                     if not self.l oadFromDict(source1):
#                         # chargement ASCII    
#                         if not self.loadWeightsASCII(source, self):
#                             # chargement XML
#                             self.loadWeightsXML(source, self)
                return res
            except:
                return False
        elif isinstance(source, (list, np.ndarray)):
            if self.weights is None:
                self.weights = np.asarray(source)
            # chargement d'un vecteur
            else:
                self.weights = source
        else:  #if isinstance(source, dict):
            return self.loadFromDict(source)

    def adaptParams(self, params):
        params = np.asarray(params)
        ddim = params.shape[0]
        if ddim == self.paramCount:
            return params
        try:
            return np.asarray([params[ind] for ind in self.wused])
        except:
            return None
        
    def adaptDispersion(self, dispersion):
        dispersion = np.asarray(dispersion)
        ddim = dispersion.shape[0]
        if ddim == self.paramCount:
            return dispersion 
        try:            
            return np.asarray([[dispersion[ii][jj] for jj in self.wused] for ii in self.wused])
        except:
            return None
        
    def adaptHalfDispersion(self, halfdispersion):
        return halfdispersion

    def loadFromDict(self, source): # ModelLib
        try:
            paramnames = None
            dispersionMatrix = None
            
            #halfDispersionMatrix = None
            for key, value in source.items():
                try:
                    if key == "propertyname":
                        self.propertyname = str(value)
                    elif key == "paramnames":
                        paramnames = list(value)
                    elif key == "params":
                        params = np.asarray(value)
                    elif key in ("outnorm", "outputnorm"):
                        self.outputnorm = value
                    elif key == "baselen":
                        self._baselen = int(value)
                    elif key == "stddev":
                        self.trainingStddev = float(value)
                    elif key == "dispersion":
                        dispersionMatrix = np.asarray(value)
                    #elif key == "halfdispersion":
                    #    halfDispersionMatrix = np.asarray(value)
                    elif key == "extraweights":
                        extraweights = value
                        nbmulti = extraweights.shape[0]
                        self._extraweights = np.zeros((nbmulti, self.paramCount))
                        for ind, ww in enumerate(extraweights):
                            self._extraweights[ind] = self.adaptParams(ww)
                            #self._extraweights[ind] = self.adaptParams(extraweights[ind])
                    elif key == "LOOweights":
                        ndim = value.ndim
                        shape = value.shape
                        if ndim == 2:
                            nbmulti = shape[0]
                            LOOweights = value
                        elif ndim == 3:
                            nbmulti = shape[1]
                            LOOweights = value[0]
                            LOOweightsMulti = value
                            self._LOOweightsMulti = np.zeros((shape[0], nbmulti, self.paramCount))
                            for matloc, mat in zip(self._LOOweightsMulti, LOOweightsMulti):
                                for ind, ww in enumerate(mat):
                                    matloc[ind] = self.adaptParams(ww)
                        self._LOOweights = np.zeros((nbmulti, self.paramCount))
                        for ind, ww in enumerate(LOOweights):
                            self._LOOweights[ind] = self.adaptParams(ww)
                            
                    elif key == "BSweights":
                        BSweights = value
                        nbmulti = BSweights.shape[0]
                        self._BSweights = np.zeros((nbmulti, self.paramCount))
                        for ind, ww in enumerate(BSweights):
                            self._BSweights[ind] = self.adaptParams(ww)
                    elif key == "extradispersion":
                        extradispersion = value
                        #nbmuti = extradispersion.shape[0]
                        self._extradispersions = np.asarray([self.adaptDispersion(mat) for mat in extradispersion])
                except: pass    
            if (paramnames is None):
                self.params = params
                self.dispersion = dispersionMatrix
                #self.halfdispersion = halfDispersionMatrix
            else:
                self.params = self.adaptParams(params)
                try:
                    source = self.adaptDispersion(dispersionMatrix)
                    self.dispersion = source
                except Exception as err:
                    pass
                #self.halfdispersion = self.adaptHalfDispersion(halfDispersionMatrix)
            return True
        except (IOError, ModelLibError) as err:
            return False
    
    @Property
    def wused(self): # ModelLib
        ls = self.paramCount
        target = np.zeros((ls,), dtype=np.int)
        target_p = c_void_p(target.ctypes.data)
        #res = 
        self.lib.getwused(target_p)
        return target
    
    @Property
    def freedom(self):
        return self.lib.getfreedom(0)
    
    def transfer(self, weights=None, inputs=None, norm=None): # ModelLib
        no = self.outputCount
        ni = self.inputCount
        if ni:
            if inputs is None:
                source_p = None
            else:
                source = np.asarray(inputs, np.double)
                source_p = c_void_p(source.ctypes.data)
        target = np.zeros((no,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        if weights is None:
            weights_p = None
        else:
            weights_s = np.asarray(weights, np.double)
            weights_p = c_void_p(weights_s.ctypes.data)
        if norm is None:
            norm_p = None
        else:
            norm_s = np.asarray(norm, np.double)
            norm_p = c_void_p(norm_s.ctypes.data)
        if ni:
            #res = 
            self.lib.transfer(weights_p, source_p, target_p, norm_p)            
        else:
            #res = 
            self.lib.transfer(weights_p, target_p, norm_p)
       
        if no == 1:
            target = target[0]
        return target
    
    def transferprime(self, weights=None, norm=None): # ModelLib
        no = self.outputCount
        nw = self.paramCount
        target = np.zeros((no,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        gradient = np.zeros((nw,), dtype=np.double)
        gradient_p = c_void_p(gradient.ctypes.data)
        if weights is None:
            weights_p = None
        else:
            weights_s = np.asarray(weights, np.double)
            weights_p = c_void_p(weights_s.ctypes.data)
        if norm is None:
            norm_p = None
        else:
            norm_s = np.asarray(norm, np.double)
            norm_p = c_void_p(norm_s.ctypes.data)
        #res = 
        self.lib.transferprime(weights_p, target_p, gradient_p, norm_p)
       
        if no == 1:
            target = target[0]
        return target, gradient
    
    def transferleverage(self, weights=None, disp=None, norm=None, style=0): # ModelLib  # default par disp
        mem = None
        externdisp = disp is not None
        if externdisp:
            mem = self.dispersion
            self.dispersion = disp
        no = self.outputCount
        target = np.zeros((no,), dtype=np.double)
        target_p = c_void_p(target.ctypes.data)
        leverage = np.zeros((no,), dtype=np.double)
        leverage_p = c_void_p(leverage.ctypes.data)
        if weights is None:
            weights_p = None
        else:
            weights_s = np.asarray(weights, np.double)
            weights_p = c_void_p(weights_s.ctypes.data)
        if norm is None:
            norm_p = None
        else:
            norm_s = np.asarray(norm, np.double)
            norm_p = c_void_p(norm_s.ctypes.data)
        if style == 1:
            # use of halfdispersion
            self.lib.transferleverage2(weights_p, target_p, leverage_p, norm_p)
        else:
            # direct use of dispersion
            self.lib.transferleverage(weights_p, target_p, leverage_p, norm_p)
       
        if no == 1:
            target = target[0]
            leverage = leverage[0]
            try:
                self.dispersion = mem
            except Exception as err:
                pass
        return target, leverage

    def transferIC(self, weights=None, inputs=None, acc=0.95, style=0, withlev=0, withhalf=0): # ModelLib
        out, leverage = self.transferleverage(weights, style=withhalf) # ici style = 1 -> methode halfdispersion
        stddev = self.trainingStddev
        df = self.freedom
        inter = interval(acc, df)
        if leverage >= 0:
            coeff = stddev * math.sqrt(leverage)
        else:
            coeff = 0
        if not style:
            if withlev == 2:
                return tuple([out] + [out + val * coeff for val in inter] + [leverage])
            elif withlev:
                return tuple([out + val * coeff for val in inter] + [leverage])
            else:
                return tuple(out + val * coeff for val in inter)
        else:
            IC =  interval(acc)[1] * coeff
            if withlev:
                return out[0], IC, leverage
            return out[0], IC

    def transferEx(self, inputs=None): # ModelLib
        try:
            outs = []
            levs = []
            for lweight, ldisp in zip(self.extraweights, self.extradispersions):     #self.dispersion = ldisp
                out, lev = self.transferleverage(lweight, disp=ldisp ,style=0)
                outs.append(out)
                levs.append(lev)
            return outs, levs            
        except AttributeError:
            return None, None
    
    def long2short(self, source):
        wused = self.wused
        ls = len(wused)
        if ls == source.shape[0]:
            return source
        sh = list(source.shape)
        dims = tuple([ls] + sh[1:])
        res = np.zeros(dims)
        for i, ind in enumerate(wused):
            res[i] = source[ind]
        return res
    
    def short2long(self, source): # ModelLib
        ls = self.paramCount
        ll = self.dimension
        if ll <= ls:
            return source
        if len(source) != ls:
            return source
        result = np.zeros((ll,))
        wused = self.wused
        for index, ind in enumerate(wused):
            result[ind] = source[index]
        return result
        
def VerifyModule(filename, datarange, modelcount):
    if not os.path.exists(filename):
        return False
    try:
        with LibManager(filename) as mgr:
            ret = mgr.codeVersionOK()
            try:
                ret = ret and hasattr(mgr, "datarange") and mgr.datarange == datarange
            except:
                ret = False
            try:
                ret = ret and hasattr(mgr, "modelCount") and mgr.modelCount == int(modelcount)
            except:
                ret = False
#            hasattr(mgr, "modelCount") and mgr.modelCount == modelcount
    except LibraryManagerError:
        ret = False
    return ret        

def codeVersionAndDynamicLinkingOK(filename, dynamiclinking):
    if not os.path.exists(filename):
        return False
    try:
        with LibManager(filename) as mgr:
            ret = mgr.codeVersionOK()
            try:  # compatibilité
                retd = mgr.dynamiclinking == dynamiclinking
            except:
                retd = False
            ret = ret and retd
    except LibraryManagerError:
        ret = False
    return ret

def codeVersionOK(filename): 
    if not os.path.exists(filename):
        return False
    try:
        with LibManager(filename) as mgr:
            ret = mgr.codeVersionOK()
    except LibraryManagerError:
        ret = False
    return ret
        
def useCodeVersionOK(filename): 
    if not os.path.exists(filename):
        return False
    #ret = False
    try:
        with LibManager(filename) as mgr:
            ret = mgr.usecodeVersionOK()
    except LibraryManagerError:
        ret = False
    return ret
    
def expectedPpty(filename, ppty, value):
    if not os.path.exists(filename):
        return False
    ret = False
    try:
        with LibManager(filename) as mgr:
            if isinstance(ppty, string_types):
                ret = mgr.getStrPropertyFromLib(ppty) == value
            else:
                for ind, (ppty1, value1) in enumerate(zip(ppty, value)):
                    if not ind:
                        ret = True
                    ret = ret and mgr.getStrPropertyFromLib(ppty1) == value1
    except LibraryManagerError:
        ret = False
    return ret

        
# if __name__ == "__main__":
#     path = "C:\Projets\GM\TestAD_2\BaseA109"
#     module = "basea109_chi00n.d ll"
#     module = os.path.join(path, module)
#     moduleold = "basea109_chi04n.dl l"
#     moduleold = os.path.join(path, moduleold)
#     
#     
#     print CC.__version__
#     print codeVersionOK(module)
#     print codeVersionOK(moduleold)
    
#------------------------------------------------------------------------------    
# if __name__ == "__main__":
#     print student.interval(0.99, 5)
#     print interval(0.99, 5)
    
#     testFileDir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..',  '..', "testFiles"))
#     libfile = os.path.join(testFileDir, "libpropane_.so")
#     gmxfile =  os.path.join(testFileDir, "LogP_base321_dgrs3_3n.gmx")
#     
#     model = ModelLib(libfile, gmxfile=gmxfile)
#     print(model)



# -*- coding: ISO-8859-1 -*-
'''
Created on 15 déc. 2017

@author: jeanluc
'''
import unittest

from . import modelutils, monalconst, monalrecords, Property, specialmath
from .datareader import _excel, check_sigs, datasource, excelsource, fileInspect
from .driver import _dataset, _driver, _driverlib, _drivermultidyn, _networkfactory, _optimizer, _parallelJobs, _workshop
from .lcode import codeconst, codewriter, codewriter, codewriterdetailC
from .library import libmanager
from .model import _link, _model, _node, _modelcontainer, _modellib, _monalfunc, _network, _network, _pseudomatrix
from .util import cleanxml, const_st, leverage, lineartoolbox, mathtoolbox, modelworkshop, monalbase, monaltoolbox, setuptoolbox, sqlite_copy_db, toolbox, utils


class Test(unittest.TestCase):


    def testName(self):
        pass

class Test_modelutils(unittest.TestCase):


    def testName(self):
        pass

class Test_monalconst(unittest.TestCase):


    def testName(self):
        pass

class Test_monalrecords(unittest.TestCase):


    def testName(self):
        pass

class Test_Property(unittest.TestCase):


    def testName(self):
        pass

class Test_specialmath(unittest.TestCase):


    def testName(self):
        pass

class Test_datareader_excel(unittest.TestCase):


    def testName(self):
        pass

class Test_datareader_check_sigs(unittest.TestCase):


    def testName(self):
        pass

class Test_datareader_datasource(unittest.TestCase):


    def testName(self):
        pass

class Test_datareader_excelsource(unittest.TestCase):


    def testName(self):
        pass

class Test_datareader_fileinspectr(unittest.TestCase):


    def testName(self):
        pass

# class Test_monal(unittest.TestCase):
# 
# 
#     def testName(self):
#         pass

class Test_driver_dataset(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_driver(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_driverlib(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_drivermultudyn(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_networkfactory(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_optimizer(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_parallelJob(unittest.TestCase):


    def testName(self):
        pass

class Test_driver_workshop(unittest.TestCase):


    def testName(self):
        pass

class Test_lcode_codeconst(unittest.TestCase):


    def testName(self):
        pass

class Test_lcode_codewriter(unittest.TestCase):


    def testName(self):
        pass

class Test_lcode_codewriterC(unittest.TestCase):


    def testName(self):
        pass

class Test_lcode_codewriterdetailC(unittest.TestCase):


    def testName(self):
        pass

class Test_library_libmabager(unittest.TestCase):


    def testName(self):
        pass

class Test_model_link(unittest.TestCase):


    def testName(self):
        pass

class Test_model_linmodel(unittest.TestCase):


    def testName(self):
        pass

class Test_model_model(unittest.TestCase):


    def testName(self):
        pass

class Test_model_modelcontainer(unittest.TestCase):


    def testName(self):
        pass

class Test_model_modellib(unittest.TestCase):


    def testName(self):
        pass

class Test_model_monalfunc(unittest.TestCase):


    def testName(self):
        pass

class Test_model_network(unittest.TestCase):


    def testName(self):
        pass

class Test_model_node(unittest.TestCase):


    def testName(self):
        pass

class Test_model_pseudomatrix(unittest.TestCase):


    def testName(self):
        pass

class Test_util_cleanxml(unittest.TestCase):


    def testName(self):
        pass

class Test_util_const_st(unittest.TestCase):


    def testName(self):
        pass

class Test_util_leverage(unittest.TestCase):


    def testName(self):
        pass

class Test_util_lineartoolbox(unittest.TestCase):


    def testName(self):
        pass

class Test_util_mathtoolbox(unittest.TestCase):


    def testName(self):
        pass

class Test_util_modelworkshop(unittest.TestCase):


    def testName(self):
        pass

class Test_util_modelbase(unittest.TestCase):


    def testName(self):
        pass

class Test_util_monaltoolbox(unittest.TestCase):


    def testName(self):
        pass

class Test_util_setuptoolbox(unittest.TestCase):


    def testName(self):
        pass

class Test_util_sqlite_copy_db(unittest.TestCase):


    def testName(self):
        pass

if __name__ == "__main__":
    unittest.main()


'''
Created on 20 févr. 2019

@author: jeanluc
metaphor.preproc.api._api_service
'''

from ...nntoolbox.utils import doNothing
from ...nntoolbox.cmd_line_dialogs import choiceQuestion, \
    valueQuestion, yesNoQuestion, chooseFields
from ...nn1.api._api_service import dialogdefaults
from ...preproc.preproc_engine import _preprocAction, preproctypelist, preprocdefaults

modulesticker = "PP"

fullpreproctypelist = preproctypelist + list(range(len(preproctypelist)))

help1="""Preproc type: 
    '' -> none, 
    'repartition' -> random probe linear function', 
    'normal -> random probe with linear + crossd-product' 
    'full -> random probe with full 2nd order'"""
arg1 = ('-pp', '--preproctype', str, "linear",fullpreproctypelist, "dico",
    help1)

help2 = "Number of random probe initialization. Default 1000"
arg2 = ('-pc', '--probecount', int, 1000, fullpreproctypelist, "dico", 
    help2)

add_arg_list = [arg1, arg2]

def preProcAction(options, doprint=doNothing, forcedefault=False):
    """Input data selection with the random probe method.
    
    Arguments:
     - options -> options namespace.
     - doprint -> print function
     - forcedefault -> force the action without question.
     
    Following the action, the data selection in options may be changed
    """
    if not 'preproc' in options.actionlist or not 'inputs' in options.grouplist:
        return
    mess = "do you want to perform inputs relevance analysis"
    valdef = int(dialogdefaults['preproc']['dopreproc'])
    accept = yesNoQuestion(mess, default=valdef, doprint=doprint, 
                           forcedefault=forcedefault)
    if accept:           
        getout = False
        doprint("running input selection")
        curyes = options.yes
        options.yes = False
        try:
            preprocres = _preprocAction(options, doprint)
            if preprocres is not None:
                options = preprocres
            else:
                getout = True
        except Exception as err:
            doprint(err)
            getout = True
        finally:
            options.yes = curyes
            doprint('api input selection done')
        if getout:
            return -1
    return 0 



'''
Created on 10 févr. 2019

@author: jeanluc
'''
import os, sys

from ._api_service import getVersionList
from ...nntoolbox.excelutils import getVbaProjectBin

def printInfo(subject='', caller=0):
    lst = getinfolist(subject, caller)
    for val in lst:
        print(val)

def getinfolist(subject='', caller=0):
    subject = subject.lower()
    result = []
    basedir = os.path.expanduser(chr(126))
    if not caller:
        basedir = os.path.join(basedir, "docker")
    if subject == 'help':
        if caller == 2:
            filename = "help/helplight.txt"
        else:
        #if caller == 2:
            filename = "help/help.txt"
        with open(os.path.join(basedir, filename), "r") as ff:
            st = ff.read().strip()
            result = [st]
    elif subject == 'license':
        with open(os.path.join(basedir, "LICENSE"), "r") as ff:
            st = ff.read().strip()
            result = [st]
    elif subject == 'description':
        with open(os.path.join(basedir, "README.txt"), "r") as ff:
            st = ff.read().strip()
            result = [st]
    elif subject == 'version':
        result = []
        with open(os.path.join(basedir, "version.txt"), "r") as ff:
            st = ff.read().strip()
            result.append("docker image : {0}".format(st))
        result.extend(getVersionList())   
    elif subject == 'env':
        result = []
        for key, val in os.environ.items(): 
            result.append("environ[{0}] = {1}".format(key, val))        
    elif subject == "vba":
        st = getVbaProjectBin()
        result = [st]
    return result
        
if __name__ == '__main__':
    action = 'version' if len(sys.argv) < 2 else sys.argv[1]
    printInfo(action)
    


# -*- coding: ISO-8859-15 -*-
#===============================================================================
# $Id: _monalfunc.py 4713 2018-01-27 10:40:41Z jeanluc $
#
#  Module  monalfunc.py
#  Projet MonalPy
#
#  Exposition des fonctions utilisées par le moteur neuronal pour les activations 
#    et les calculs de coûts.
#
#  Author: Jean-Luc PLOIX  -  NETRAL
#  Decembre 2009
#
# Copyright 2016 Jean-Luc PLOIX
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#===============================================================================

"""Module monalfunc pour Monal Python

   fonctions mathématiques,
   dictionnaires de fonctions
""" 

from numpy import arctan, sqrt, pi, tanh, sin, cos, exp, log, cosh, any
from math import erf, asinh
from ..monalconst import NONE, SQUAREDELTA, SQUARELOGDELTA, LEASTSQUAREDELTA, \
    WEIGHTEDSQUAREDELTA, UNCLASSIFIED, WEIGHTEDCLASSIFICATION, RELATIVESQUAREDELTA, \
    GAUSSSQUAREDELTA, CROSSEDENTROPY, EXPSQUAREDELTA, \
    IDENTITY, UNIT, TANH, STEP, GAUSS, SINE, EXP, QUADRATIC, ATAN, MOMENT0, MOMENT1, \
    REVERSE, LOG, CUBIC, SIGMOID, COMPLEMENT, ROOT, ARCSH, SQR, COS, ERF


# fonctions utiles
sqr = lambda X: X*X
cubic = lambda X: X*X*X
gaussNorm = lambda X: exp(-0.5*X*X)/sqrt(2*pi)
#gaussNorm = lambda X: exp(-0.5*X*X)/sqrt(2*pi)

def erorFunction(X):
    """fonction integrale de la gaussienne normalisee"""
    if X == 0:
        return 0.5
    if X < 0:
        return 1 - erorFunction(-X)
    T = 1/(1 + 0.2316419*X)
    R = ((((1.330274429*T - 1.821255978)*T + 1.781477937)*T - 0.356563782)*T + 
         0.31938153)*T
    return 1 - R*gaussNorm(X)

#===========================================================================
# Functors de cout
#===========================================================================
class basecost(object):
    
    def __init__(self):
        self.clear()
    def clear(self): pass
    def __str__(self): return
    def __repr__(self): return
    def __call__(self, A, B, C=1): return None
    def derive(self, A, B, C=1): return None
    
class nullcost(basecost):
    def __str__(self): return "null"
    def __repr__(self): return "null"
    def __call__(self, A, B, C=1): return 0
    def derive(self, A, B, C=1): return 0

class quadcost(basecost):
    def __str__(self): return "deltasquare"
    def __repr__(self): return "delta square"
    def __call__(self, A, B, C=1): return sqr(A-B)/2
    def derive(self, A, B, C=1): return A-B

class logquadcost(basecost):
    def __str__(self): return "logdeltasquare"
    def __repr__(self): return "logarithm delta square"
    def __call__(self, A, B, C=1): return sqr(log(float(A)/B))/2
    def derive(self, A, B, C=1): return log(float(A) / B)/A

class minquadcost(basecost):
    def __str__(self): return "leastdeltasquare"
    def __repr__(self): return "least delta square"
    def __call__(self, A, B, C=1): return sqrt(1 + sqr(abs((A - B) / C))) - 1
    def derive(self, A, B, C=1): return (A - B) / C/C / sqrt(1 + sqr((A - B)/C))

class weightquadcost(basecost):
    def __str__(self): return "weighteddeltasquare"
    def __repr__(self): return "weighted delta square"
    def __call__(self, A, B, C=1): return sqr(A - B) / 2 / (1 + sqr(B / C))
    def derive(self, A, B, C=1): return (A - B) / (1 + sqr(B / C))

class mispositioncost(basecost):
    def __str__(self): return "misclassified"
    def __repr__(self): return "misclassified"
    def __call__(self, A, B, C=1): return int((A - C) * (B - C) < 0)

class relquadcost(basecost):
    def __str__(self): return "reldeltasquare"
    def __repr__(self): return "relative delta square"
    def __call__(self, A, B, C=1): return sqr(A / B - 1) / 2
    def derive(self, A, B, C=1): return (A / B - 1) / B

class gaussquadcost(basecost):
    def __str__(self): return "gaussdeltasquare"
    def __repr__(self): return "gaussian delta square"
    def __call__(self, A, B, C=1): return sqr(A - B) / 2 * exp(-sqr(B / C))
    def derive(self, A, B, C=1): return (A - B) * exp(-sqr(B / C))

def crossentropycost(basecost):
    def __str__(self): return "crossentropy"
    def __repr__(self): return "cross entropy"
    def __call__(A, B, C=1): return B*(1-A) + A*(1-B)
#        if B == 1: return 1 - A
#        return A - B
    def derive(A, B, C=1): return 1 - 2*B
#        if B == 1: return -1
#        return 1

class expquadcost(basecost):
    def __str__(self): return "expdeltasquare"
    def __repr__(self): return "exponent delta square"
    def __call__(self, A, B, C=1): return 10 * (exp(sqr(A - B) / 2) - 1)
    def derive(self, A, B, C=1): return 10 * (exp(sqr(A - B) / 2) - 1)
    
class lateralcost(basecost):
    def __str__(self): return "weightclassification"
    def __repr__(self): return "weighted classification"
    def __call__(self, A, B, C=1):
        X = A - C
        V = sqrt(1 + sqr(X))
        if B < C: return sqr((V + X) / 2)
        else: return sqr((V - X) / 2)
    def derive(self, A, B, C=1):
        X = A - C;
        V = sqrt(1 + sqr(X))
        if B < C: return sqr((X + V)) / V / 2
        else: return -sqr((X - V)) / V / 2
        
#===========================================================================
# Fin des functors de cout
#===========================================================================
# Les fonctions de coût. 
classCostIndex = {
    NONE: nullcost,                      # 0
    SQUAREDELTA: quadcost,               # 1
    SQUARELOGDELTA: logquadcost,         # 2
    LEASTSQUAREDELTA: minquadcost,       # 3
    WEIGHTEDSQUAREDELTA: weightquadcost, # 4
    UNCLASSIFIED: mispositioncost,       # 5
    WEIGHTEDCLASSIFICATION: lateralcost, # 6
    RELATIVESQUAREDELTA: relquadcost,    # 7
    GAUSSSQUAREDELTA: gaussquadcost,     # 8
    CROSSEDENTROPY: crossentropycost,    # 9
    EXPSQUAREDELTA: expquadcost          # 10
    }

costderivelist = {}

# la clé est le nom du functor, un functor peut apparaitre plusieurs fois.
classCostDict = {
    "": nullcost,
    "Null": nullcost,
    "deltasquare": quadcost,
    "Delta carré": quadcost,
    "logdeltasquare": logquadcost,
    "Delta log carré": logquadcost,
    "lesslogdelta": minquadcost,
    "Delta carré minoré": minquadcost,
    "weighteddeltasquare": weightquadcost,
    "weightlogdelta": weightquadcost,
    "Delta carré pondéré": weightquadcost,
    "mispositionned": mispositioncost,
    "misclassified": mispositioncost,
    "Classement incorrect": mispositioncost,
    "weightposition": lateralcost,
    "weightclassification": lateralcost,
    "Classement pondéré": lateralcost,
    "reldeltasquare": relquadcost,
    "relativedeltaquad": relquadcost,
    "Delta carré relatif": relquadcost,
    "gaussdeltasquare": gaussquadcost,
    "Delta carré gauss": gaussquadcost,
    "crossentropy": crossentropycost,
    "Entropie croisée": crossentropycost,
    "expdeltasquare": expquadcost,
    "exponentdeltasquare": expquadcost,
    "Exponentielle delta carré": expquadcost
    } 

def getCostClass(index):
    try: return classCostDict[index.lower()]
    except: return classCostIndex[index]


#===========================================================================
# Functors d'activation
#===========================================================================
class baseactiv(object):
    index = -1
    def __init__(self):
        self.clear()
    
    def clear(self):
        self._X = None
        self._XD = None
        self._XD2 = None
        self._Y = None
        self._YD = None
        self._YD2 = None
    
    def template(self): return ""
    
    def formula(self, expr):
        try: return self.template()% expr
        except: pass
        try: return self.template()% (expr, expr)
        except: pass
        try: return self.template()% (expr, expr, expr)
        except: pass
        try: return self.template()% (expr, expr, expr, expr)
        except: pass
        return self.self.template()
    
    def __call__(self, value, withDerive=False):
        if withDerive:
            ff = self.func(value)
            if hasattr(self, 'funcDY'):
                fd = self.funcDY(value, ff)
            else:
                fd = self.funcD(value)
            return ff, fd
        try:
            self._X = value[:]
            self._Y = self.func(value)[:]
        except:
            self._X = value
            self._Y = self.func(value)
        return self._Y

#     def funcAndDerive(self, value):
#         ff = self.func(value)
#         if hasattr(self, 'funcDY'):
#             fd = self.funcDY(value, ff)
#         else:
#             fd = self.funcD(value)
#         return ff, fd
    
    def derive(self, value):
        if hasattr(self, 'funcDY') and equal(self._XD, value):
            self._YD = self.funcDY(self._X, self._Y)
        else:
            self._YD = self.funcD(value)
        return self._YD

    def derive2(self, value):
        self._XD2 = value
        if hasattr(self, 'funcD2D') and (self._XD == value):
            self._YD2 = self.funcD2D(self._XD, self._YD)
        elif hasattr(self, 'funcD2Y') and (self._X == value):
            self._YD2 = self.funcD2Y(self._X, self._Y)
        else:
            self._YD2 = self.funcD2(value)           
        return self._YD2

#@singleton
class identityactiv(baseactiv):
    index = 0
    def __str__(self): return "ident"
    def __repr__(self): return "identity"
    def template(self): return "%s"
    def __call__(self, X): return X
    def derive(self, X): return 1.
    def derive2(self, X): return 0.

class oneactiv(baseactiv):
    index = 1
    def __str__(self): return "1"
    def __repr__(self): return "fixed one"
    def template(self): return "1"
    def __call__(self, X): return 1.
    def derive(self, X): return 0.
    def derive2(self, X): return 0.

class stepactiv(baseactiv):
    index = 2
    def __str__(self): return "heaviside"
    def __repr__(self): return "heaviside step"
    def template(self): return "heaviside(%s)"
    def __call__(self, X): 
        if X < 0: return 0.
        return 1.

class tanhactiv(baseactiv):
    index = 3
    def __str__(self): return "tanh"
    def __repr__(self): return "hyperbolic tangent"
    def template(self): return "tanh(%s)"
    def func(self, X): return tanh(X)
    def funcD(self, X): return 1 / sqr(cosh(X))
    def funcDY(self, X, Y): return 1 - Y*Y
    def funcDY2(self, Y): return 1 - Y*Y 
    def funcD2(self, X): return -2*tanh(X)/sqr(cosh(X))
    def funcD2Y(self, X, Y): return -2*Y*(1-Y*Y)

class gaussactiv(baseactiv):
    index = 4
    def __str__(self): return "gauss"
    def __repr__(self): return "gaussian"
    def template(self): return "exp(-%s*%s)"
    def func(self, X): return exp(-X*X)
    def funcD(self, X): return -2*X*exp(-X*X)
    def funcDY(self, X, Y): return -2*X*Y
    def funcD2(self, X): return 2*(2*X*X-1)*exp(-X*X)
    def funcD2Y(self, X, Y): return 2*(2*X*X-1)*Y
        
class sineactiv(baseactiv):
    index = 5
    def __str__(self): return "sin"
    def __repr__(self): return "sine"
    def template(self): return "sin(%s)"
    def func(self, X): return sin(X)
    def funcD(self, X): return cos(X)
    def funcD2(self, X): return -sin(X)
    def funcD2Y(self, X, Y): return -Y
        
class expactiv(baseactiv):
    index = 6
    def __str__(self): return "exp"
    def __repr__(self): return "exponential"
    def template(self): return "exp(%s)"
    def func(self, X): return exp(X)
    def funcD(self, X): return exp(X)
    def funcDY(self, X, Y): return Y
    def funcD2(self, X): return exp(X)
    def funcD2Y(self, X, Y): return Y
    def funcD2D(self, X, YD): return YD
        
class quadraticactiv(baseactiv):
    index = 7
    def __str__(self): return "quadratic"
    def __repr__(self): return "half square"
    def template(self): return "%s*%s/2"
    def func(self, X): return X*X/2
    def derive(self, X): return X
    def derive2(self, X): return 1
        
class atanactiv(baseactiv):
    index = 8
    def __str__(self): return "atan"
    def __repr__(self): return "argument tangent"
    def template(self): return "atan(%s)"
    def func(self, X): return arctan(X)
    def funcD(self, X): return 1/(1 + X*X)
    def funcD2(self, X): return -2*X/sqr(1 + X*X)

class moment0activ(baseactiv):
    index = 9
    def __str__(self): return "m0"
    def __repr__(self): return "order 0 moment"
    def template(self): return "(1 - %s*%s)/(1 + %s*%s)"   
    def func(self, X):
        Z = sqr(X)
        return (1 - Z)/(1 + Z)
    def funcD(self, X): return -4*X/sqr(1 + sqr(X))
    def funcD2(self, X): return -4*(1 - 3*sqr(X))/cubic(1 + sqr(X))

class moment1activ(baseactiv):
    index = 10
    def __str__(self): return "m1"
    def __repr__(self): return "order 1 moment"
    def template(self): return "%s/(1 + %s*%s)"
    def func(self, X): return X/(1 + X*X)
    def funcD(self, X): return (1-sqr(X))/sqr(1 + sqr(X))
    def funcD2(self, X): return 2*X*(-3 + X*X)/cubic(1 + X*X)

class reverseactiv(baseactiv):
    index = 11
    def __str__(self): return "reverse"
    def __repr__(self): return "reverse"
    def template(self): return "1/%s"
    def func(self, X): return 1/X
    def funcD(self, X): return -1/(X*X)
    def funcDY(self, X, Y): return -Y*Y
    def funcD2(self, X): return 2/(X*X*X)
    def funcD2Y(self, X, Y): return 2*cubic(Y)

class lognactiv(baseactiv):
    index = 12
    def __str__(self): return "log"
    def __repr__(self): return "neperian logarithm"
    def template(self): return "log(%s)"
    def func(self, X): return log(X)
    def funcD(self, X): return 1/X
    def funcD2(self, X): return -1/(X*X)

class cubicactiv(baseactiv):
    index = 13
    def __str__(self): return "cubic"
    def __repr__(self): return "cubic"
    def template(self): return "%s*%s*%s"
    def func(self, X): return X*X*X
    def funcD(self, X): return 3*X*X
    def funcD2(self, X): return 6*X

class sigmoidactiv(baseactiv):
    index = 14
    def __str__(self): return "sig"
    def __repr__(self): return "sigmoid"
    def template(self): return "0.5*(1 + tanh(%s))"
    def func(self, X): return 0.5*(1 + tanh(X))
    def funcD(self, X): return 1 / (2*sqr(cosh(X)))
    def funcDY(self, X, Y): return 2*Y*(1-Y)
    def funcD2(self, X): return -tanh(X)/sqr(cosh(X))

class complement1activ(baseactiv):
    index = 15
    def __str__(self): return "complement"
    def __repr__(self): return "complement to 1"
    def template(self): return "1 - %s"
    def __call__(self, X): return 1-X
    def derive(self, X): return -1
    def derive2(self, X): return 0

class rootactiv(baseactiv):
    index = 16
    def __str__(self): return "sqrt"
    def __repr__(self): return "square root"
    def template(self): return "sqrt(%s)"
    def func(self, X): return sqrt(X)
    def funcD(self, X): return 1/(2*sqrt(X))
    def funcDY(self, X, Y): return 1/(2*Y)
    def funcD2(self, X): return -0.25/cubic(sqrt(X))
    def funcD2Y(self, X, Y): return -0.25/cubic(Y)

class arcsinhactiv(baseactiv):
    index = 17
    def __str__(self): return "asinh"
    def __repr__(self): return "hyperbolic sine argument"
    def template(self): return "asinh(%s)"
    def func(self, X): return asinh(X) 
    def funcD(self, X): return 1/sqrt(1 + sqr(X))
    def funcD2(self, X): return -X/sqrt(cubic(1 + sqr(X)))

class sqractiv(baseactiv):
    index = 18
    def __str__(self): return "sqr"
    def __repr__(self): return "square"
    def template(self): return "%s*%s"
    def func(self, X): return X*X
    def funcD(self, X): return 2*X
    def derive2(self, X): return 2
    
class cosactiv(baseactiv):
    index = 19
    def __str__(self): return "cos"
    def __repr__(self): return "cosine"
    def template(self): return "cos(%s)"
    def func(self, X): return cos(X)
    def funcD(self, X): return -sin(X)
    def funcD2(self, X): return -cos(X)
    def funcD2Y(self, X, Y): return -Y

class erfactiv(baseactiv):
    index = 20
    def __str__(self): return "erf"
    def __repr__(self): return "error activ"
    def formula(self): return "erf(%s)"
    def func(self, X): return 0.5 + 0.5*erf(X)  #erorFunction(X)
    def funcD(self, X): return gaussNorm(X)
    def funcD2(self, X): return -X*gaussNorm(X) 
    def funcD2D(self, X, YD): return -X*YD

#===========================================================================
# Fin des functors d'activation
#===========================================================================
classActivIndex = {
    IDENTITY: identityactiv,        # 0
    UNIT: oneactiv,                 # 1
    TANH: tanhactiv,                # 2
    STEP: stepactiv,                # 3
    GAUSS: gaussactiv,              # 4
    SINE: sineactiv,                # 5
    EXP: expactiv,                  # 6
    QUADRATIC: quadraticactiv,      # 7
    ATAN: atanactiv,                # 8
    MOMENT0: moment0activ,          # 9
    MOMENT1: moment1activ,          # 10
    REVERSE: reverseactiv,          # 11
    LOG: lognactiv,                 # 12
    CUBIC: cubicactiv,              # 13
 #   QUASIABS: ,                     # 14
    SIGMOID: sigmoidactiv,          # 15
    COMPLEMENT: complement1activ,  # 16
    ROOT: rootactiv,                # 17
    ARCSH: arcsinhactiv,            # 18
    SQR: sqractiv,                  # 19
    COS: cosactiv,                  # 20
    ERF: erfactiv                    # 21
}

classActivDict = {
    "": identityactiv,
    "ident": identityactiv,
    "identity": identityactiv,
    "unit": oneactiv,
    "tanh": tanhactiv,
    "gauss": gaussactiv,
    "sin": sineactiv,
    "sine": sineactiv,
    "exp": expactiv,
    "exponent": expactiv,
    "halfsquare": quadraticactiv,
    "quadratic": quadraticactiv,
    "atan": atanactiv,
    "arctan": atanactiv,
    "m0": moment0activ,
    "m1": moment1activ,
    "inv": reverseactiv,
    "inverse": reverseactiv,
    "reverse": reverseactiv,
    "ln": lognactiv,
    "cube6": cubicactiv,
    "sig": sigmoidactiv,
    "sigmoid": sigmoidactiv,
    "comp1": complement1activ,
    "sqrt": rootactiv,
    "arcsh": arcsinhactiv,
    "arcsinh": arcsinhactiv,
    "asinh": arcsinhactiv,
    "erf": erfactiv,
    "errorfct": erfactiv,
    "sqr": sqractiv,
    "cos": cosactiv,
    "cosine": cosactiv}

def getActivClass(index):
    try: return classActivDict[index.lower()]
    except: return classActivIndex[index]
    
def getIndexFromActivClass(aclass):
    if isinstance(aclass, str):
        aclass = classActivDict[aclass]
    for item in list(classActivIndex.items()):
        if item[1] == aclass:
            return item[0]
    return None

def deriveDF(func, X, delta):
    return (func(X+delta/2)-func(X-delta/2))/delta

def derive2DF(func, X, delta):
    return (func(X+delta) + func(X-delta) - 2*func(X))/(delta*delta)

if __name__ == "__main__":
    import unittest
    class TestFunctions(unittest.TestCase):
        
        def __init__(self, *args, **kwargs):
            super(TestFunctions, self).__init__(*args, **kwargs)
            
        def testErf(self):
            F = erfactiv()
            X = 10.0
            self.assertAlmostEqual(erorFunction(X), F(X), 8)
            X = 0.1
            self.assertAlmostEqual(erorFunction(X), F(X), 3)
    
            
    unittest.main()
    
def equal(X, Y):
    try:
        temp = X - Y
        return not any(temp)
    except:
        return X == Y
        


# -*- coding: ISO-8859-1 -*-
#-------------------------------------------------------------------------------
# $Id$
#
# Copyright 2016 Jean-Luc PLOIX
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-------------------------------------------------------------------------------
'''
Created on 14 december 2018

@author: jeanluc
'''
# import sys
# from . import  nn_run
# 
# nn_run.run(sys.argv)
#from metaphor.monal.util.monaltoolbox import yesNoQuestion
#from metaphor import chem_gm
from metaphor.nn1.api.run import nn_run
import sys, os

# res = 
nn_run.supervisor()

# from argparse import Namespace
# if isinstance(res, Namespace):
#     print("results in :", res.metaphorResult[0])
print("All Done")